@@ -173,6 +173,9 @@ class MockRBD(object):
         def clone(self, *args, **kwargs):
             raise NotImplementedError()
 
+        def trash_move(self, *args, **kwargs):
+            pass
+
     RBD_FEATURE_LAYERING = 1
 
 
@@ -633,37 +636,51 @@ class TestStore(base.StoreBaseTest,
         with mock.patch.object(MockRBD.Image, 'list_children') as mocked:
             mocked.return_value = True
 
-            self.assertRaises(exceptions.InUseByStore,
-                              self.store._delete_image,
-                              'fake_pool', self.location.image,
-                              snapshot_name='snap')
+            self.store._delete_image('fake_pool',
+                                     self.location.image,
+                                     snapshot_name='snap')
 
     def test_delete_image_w_snap_exc_image_has_snap(self):
         def _fake_remove(*args, **kwargs):
             self.called_commands_actual.append('remove')
             raise MockRBD.ImageHasSnapshots()
 
+        mock.patch.object(MockRBD.RBD, 'trash_move').start()
+
         with mock.patch.object(MockRBD.RBD, 'remove') as remove:
             remove.side_effect = _fake_remove
-            self.assertRaises(exceptions.HasSnapshot, self.store._delete_image,
-                              'fake_pool', self.location.image)
+            self.store._delete_image('fake_pool',
+                                     self.location.image)
 
             self.called_commands_expected = ['remove']
 
+        MockRBD.RBD.trash_move.assert_called_once_with(mock.ANY, 'fake_image')
+
+    def test_delete_image_w_snap_exc_image_has_snap_2(self):
+        def _fake_remove(*args, **kwargs):
+            self.called_commands_actual.append('remove')
+            raise MockRBD.ImageHasSnapshots()
+
+        mock.patch.object(MockRBD.RBD, 'trash_move',
+                          side_effect=MockRBD.ImageBusy).start()
+
+        with mock.patch.object(MockRBD.RBD, 'remove') as remove:
+            remove.side_effect = _fake_remove
+            self.assertRaises(exceptions.InUseByStore,
+                              self.store._delete_image,
+                              'fake_pool',
+                              self.location.image)
+
+            self.called_commands_expected = ['remove']
+
+        MockRBD.RBD.trash_move.assert_called_once_with(mock.ANY, 'fake_image')
+
     def test_get_partial_image(self):
         loc = g_location.Location('test_rbd_store', rbd_store.StoreLocation,
                                   self.conf, store_specs=self.store_specs)
         self.assertRaises(exceptions.StoreRandomGetNotSupported,
                           self.store.get, loc, chunk_size=1)
 
-    @mock.patch.object(MockRados.Rados, 'connect')
-    def test_rados_connect_timeout(self, mock_rados_connect):
-        socket_timeout = 1
-        self.config(rados_connect_timeout=socket_timeout)
-        self.store.configure()
-        with self.store.get_connection('conffile', 'rados_id'):
-            mock_rados_connect.assert_called_with(timeout=socket_timeout)
-
     @mock.patch.object(MockRados.Rados, 'connect', side_effect=MockRados.Error)
     def test_rados_connect_error(self, _):
         rbd_store.rados.Error = MockRados.Error
@@ -700,3 +717,67 @@ class TestStore(base.StoreBaseTest,
         self.assertEqual(self.called_commands_expected,
                          self.called_commands_actual)
         super(TestStore, self).tearDown()
+
+    @mock.patch('oslo_utils.eventletutils.is_monkey_patched')
+    def test_create_image_in_native_thread(self, mock_patched):
+        mock_patched.return_value = True
+        # Tests that we use non-0 features from ceph.conf and cast to int.
+        fsid = 'fake'
+        features = '3'
+        conf_get_mock = mock.Mock(return_value=features)
+        conn = mock.Mock(conf_get=conf_get_mock)
+        ioctxt = mock.sentinel.ioctxt
+        name = '1'
+        size = 1024
+        order = 3
+        fake_proxy = mock.MagicMock()
+        fake_rbd = mock.MagicMock()
+
+        with mock.patch.object(rbd_store.tpool, 'Proxy') as tpool_mock, \
+                mock.patch.object(rbd_store.rbd, 'RBD') as rbd_mock:
+            tpool_mock.return_value = fake_proxy
+            rbd_mock.return_value = fake_rbd
+            location = self.store._create_image(
+                fsid, conn, ioctxt, name, size, order)
+            self.assertEqual(fsid, location.specs['fsid'])
+            self.assertEqual(rbd_store.DEFAULT_POOL, location.specs['pool'])
+            self.assertEqual(name, location.specs['image'])
+            self.assertEqual(rbd_store.DEFAULT_SNAPNAME,
+                             location.specs['snapshot'])
+
+        tpool_mock.assert_called_once_with(fake_rbd)
+        fake_proxy.create.assert_called_once_with(ioctxt, name, size, order,
+                                                  old_format=False, features=3)
+
+    @mock.patch('oslo_utils.eventletutils.is_monkey_patched')
+    def test_delete_image_in_native_thread(self, mock_patched):
+        mock_patched.return_value = True
+        fake_proxy = mock.MagicMock()
+        fake_rbd = mock.MagicMock()
+        fake_ioctx = mock.MagicMock()
+
+        with mock.patch.object(rbd_store.tpool, 'Proxy') as tpool_mock, \
+                mock.patch.object(rbd_store.rbd, 'RBD') as rbd_mock, \
+                mock.patch.object(self.store, 'get_connection') as mock_conn:
+
+            mock_get_conn = mock_conn.return_value.__enter__.return_value
+            mock_ioctx = mock_get_conn.open_ioctx.return_value.__enter__
+            mock_ioctx.return_value = fake_ioctx
+            tpool_mock.return_value = fake_proxy
+            rbd_mock.return_value = fake_rbd
+
+            self.store._delete_image('fake_pool', self.location.image)
+
+            tpool_mock.assert_called_once_with(fake_rbd)
+            fake_proxy.remove.assert_called_once_with(fake_ioctx,
+                                                      self.location.image)
+
+    @mock.patch.object(rbd_store, 'rbd')
+    @mock.patch.object(rbd_store, 'tpool')
+    @mock.patch('oslo_utils.eventletutils.is_monkey_patched')
+    def test_rbd_proxy(self, mock_patched, mock_tpool, mock_rbd):
+        mock_patched.return_value = False
+        self.assertEqual(mock_rbd.RBD(), self.store.RBDProxy())
+
+        mock_patched.return_value = True
+        self.assertEqual(mock_tpool.Proxy.return_value, self.store.RBDProxy())
